Package: HelloRanges
Type: Package
Title: Introduce *Ranges to bedtools users
Version: 1.38.0
Author: Michael Lawrence
Maintainer: Michael Lawrence <lawremi@gmail.com>
Description: Translates bedtools command-line invocations to R code
        calling functions from the Bioconductor *Ranges infrastructure.
        This is intended to educate novice Bioconductor users and to
        compare the syntax and semantics of the two frameworks.
License: GPL (>= 2)
Imports: docopt, stats, tools, utils
Depends: methods, BiocGenerics, S4Vectors (>= 0.17.39), IRanges (>=
        2.13.12), GenomicRanges (>= 1.31.10), Biostrings (>= 2.41.3),
        BSgenome, GenomicFeatures (>= 1.31.5), VariantAnnotation (>=
        1.19.3), Rsamtools, GenomicAlignments (>= 1.15.7), rtracklayer
        (>= 1.33.8), Seqinfo, SummarizedExperiment, BiocIO
Suggests: GenomeInfoDb, HelloRangesData, BiocStyle, RUnit,
        TxDb.Hsapiens.UCSC.hg19.knownGene
biocViews: Sequencing, Annotation, Coverage, GenomeAnnotation,
        DataImport, SequenceMatching, VariantAnnotation
Config/pak/sysreqs: make libbz2-dev liblzma-dev libpng-dev libxml2-dev
        libssl-dev xz-utils zlib1g-dev
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 12:44:32 UTC
RemoteUrl: https://github.com/bioc/HelloRanges
RemoteRef: RELEASE_3_23
RemoteSha: 66a66acde0a32c7829bc9e3f11e4c6c0a57c7581
NeedsCompilation: no
Packaged: 2026-05-04 09:13:02 UTC; root
Built: R 4.6.0; ; 2026-05-04 09:58:26 UTC; unix
